section .text
global get_esp
global get_ebp
global get_eip
global set_esp
global set_ebp
global set_eip
global switch_context

; Get ESP (Stack Pointer)
get_esp:
    mov eax, esp ; 直接返回 esp 值
    ret

; Get EBP (Base Pointer)
get_ebp:
    mov eax, ebp ; 直接返回 ebp 值
    ret

; Get EIP (Instruction Pointer)
get_eip:
    mov eax, [esp] ; 返回调用者的返回地址作为 EIP
    ret

; Set ESP (参数在 [esp + 4])
set_esp:
    push ebp
    mov ebp, esp
    mov eax, [ebp + 8] ; 获取参数
    mov esp, eax ; 设置新的 esp
    pop ebp
    ret

; Set EBP (参数在 [esp + 4])
set_ebp:
    push ebp
    mov ebp, esp
    mov eax, [ebp + 8] ; 获取参数
    mov ebp, eax ; 设置新的 ebp
    pop ebp
    ret

; 上下文切换函数
global switch_context
switch_context: ; void switch_context(uint32_t new_esp, uint32_t new_ebp, uint32_t new_eip)
    push ebp
    mov ebp, esp

; 保存当前寄存器状态
    push ebx
    push esi
    push edi

; 获取新的上下文参数
    mov eax, [ebp + 8] ; new_esp
    mov ebx, [ebp + 12] ; new_ebp
    mov ecx, [ebp + 16] ; new_eip

; 切换栈和基址指针
    mov esp, eax
    mov ebp, ebx

; 恢复新上下文的寄存器
    pop edi
    pop esi
    pop ebx

; 跳转到新的执行点
    jmp ecx