section .data
global page_directory        ; 声明全局变量 page_directory，供其他模块使用
extern os_alloc_page         ; 声明外部函数 os_alloc_page，在 C 文件中实现

align 4096
page_directory:
    times 1024 dd 0          ; 定义页目录表，占用1024项，每项初始化为0

align 4096
page_table:
    times 1024 dd 0          ; 定义第一个页表，占用1024项，每项初始化为0

section .text
global setup_paging          ; 声明全局函数 setup_paging，供其他模块调用
global alloc_page_table      ; 声明全局函数 alloc_page_table，供其他模块调用

setup_paging:
    ; 初始化第一个页表 - 映射内存的前4MB
    mov ecx, 0               ; 用于循环计数的寄存器，初始化为0
    mov edx, page_table      ; edx 指向页表的起始地址
    mov eax, 0x3             ; 设置页表项标志：Present + Writable
.map_pt:
    mov [edx + ecx * 4], eax ; 将当前页的物理地址和标志写入页表项
    add eax, 0x1000          ; 更新到下一个页的物理地址（4KB对齐）
    inc ecx                  ; 循环计数器加1
    cmp ecx, 1024            ; 判断是否已经填充1024项（4MB）
    jl .map_pt               ; 如果未完成，继续循环

    ; 设置页目录的第一项，指向第一个页表
    mov eax, page_table      ; eax 指向第一个页表
    or eax, 0x3              ; 设置标志：Present + Writable
    mov [page_directory], eax ; 将页表地址写入页目录的第一项

    ; 将页目录地址加载到 CR3 寄存器
    mov eax, page_directory  ; eax 指向页目录
    mov cr3, eax             ; 将页目录基地址加载到 CR3

    ; 通过设置 CR0 的 PG 位启用分页
    mov eax, cr0             ; 读取 CR0 寄存器
    or eax, 0x80000000       ; 设置 PG 位（启用分页）
    mov cr0, eax             ; 写回 CR0 寄存器

    ret                      ; 返回调用点

alloc_page_table:
    ; 为新的页表分配一个页面
    call os_alloc_page       ; 调用外部函数 os_alloc_page 分配一页内存
    mov [eax], dword 0       ; 将分配的页面清零
    ret                      ; 返回调用点
