#ifndef KERNEL_H
#define KERNEL_H

#include "types.h"         // 包含类型定义
#include "video.h"         // 包含视频输出相关的声明
#include "interrupt.h"     // 包含中断处理相关的声明
#include "memory.h"        // 包含内存管理相关的声明
#include "sched.h"         // 包含调度相关的声明
#include "syscall.h"       // 包含系统调用相关的声明
#include "io.h"            // 包含输入/输出相关的声明

// 键盘端口号定义
#define KEYBOARD_DATA_PORT 0x60   // 键盘数据端口
#define KEYBOARD_STATUS_PORT 0x64 // 键盘状态端口

// 键盘状态位定义
#define KEYBOARD_OUTPUT_FULL 0x01 // 输出缓冲区已满

// 键盘扫描码定义
#define KEY_SHIFT 0x2A            // 左Shift键扫描码
#define KEY_SHIFT_RELEASE 0xAA    // 左Shift键释放扫描码

// 函数声明
void get_input(char *buffer, int max_len);            // 从键盘获取输入，保存到缓冲区中
int eval_expression(const char *expr);               // 解析并计算表达式
void print_int(int num);                              // 打印整数到屏幕
char getchar();                                       // 从键盘获取一个字符
void putchar(char c);                                 // 输出一个字符到屏幕
char scancode_to_ascii(unsigned char scancode, int shift_pressed); // 将扫描码转换为ASCII字符

// 内核初始化相关
void kmain(void);                // 内核主函数

// 系统初始化相关
void clear_screen(void);         // 清空屏幕（视频内存）

// 系统服务相关
void log_message(const char *msg);   // 将消息记录到屏幕
void print_int(int num);             // 将整数输出到屏幕

// 系统时间与调度相关
void schedule(void);                // 进程调度函数
void timer_interrupt_handler(void); // 用于调度的定时器中断处理函数
void init_scheduler(void);          // 初始化调度器

#endif // KERNEL_H
