#ifndef MEMORY_H
#define MEMORY_H

#include "types.h"   // 包含通用类型定义
#include "utils.h"   // 包含工具函数和宏定义

// 页表项（PTE）标志位定义
#define PTE_PRESENT  0x1      // 页面在内存中
#define PTE_WRITE    0x2      // 页面可写
#define PTE_USER     0x4      // 用户级访问权限
#define PTE_PWT      0x8      // 页面级写通（Write-through）
#define PTE_PCD      0x10     // 页面级缓存禁用（Cache Disable）
#define PTE_ACCESSED 0x20     // 页面已被访问
#define PTE_DIRTY    0x40     // 页面已被修改
#define PTE_ATTRIB   0x80     // 额外属性
#define MAX_TIME 0xFFFFFFFFFFFFFFFFULL // 用于时间比较的最大值
#define PAGE_LOCKED (1ULL << 63) // 使用最高位表示页面被锁定

#define PAGE_SIZE 0X1000      // 页面大小为4KB

// 外部汇编函数声明，用于设置分页
extern void setup_paging(void);

// 内存管理函数
void memory_init(void);           // 初始化内存管理系统
void* alloc_page(void);           // 分配单个物理页面
void* os_alloc_page(void);        // 操作系统分配物理页面
void free_page(void* ptr);        // 释放之前分配的物理页面

// LRU（最近最少使用）算法支持函数
void update_page_access_time(uint64_t page_index);  // 更新页面的访问时间
void lru_evict_page(void);       // 驱逐最近最少使用的页面

// 位示图，用于表示空闲页面
extern uint64_t *free_page_bitmap; // 指向空闲页面位示图
extern uint64_t total_pages;       // 内存中的页面总数

// 用于记录页面最后访问时间的数组
extern uint64_t *page_access_time;

#endif /* MEMORY_H */
