#ifndef SYSCALL_H
#define SYSCALL_H

#include "types.h"   // 包含通用类型定义

// 获取寄存器值的宏定义
#define GET_REGISTER(reg, var) \
    asm volatile(GET_##reg : "=r" (var)) // 使用汇编获取寄存器值并存入变量

// 系统调用号定义
#define SYSCALL_WRITE    0   // 系统调用：写操作
#define SYSCALL_EXIT     1   // 系统调用：退出
#define SYSCALL_READ     2   // 系统调用：读操作
#define SYSCALL_ALLOC    3   // 系统调用：分配内存
#define SYSCALL_FREE     4   // 系统调用：释放内存
#define SYSCALL_SLEEP    5   // 系统调用：休眠

// 函数声明
void syscall_handler(void);    // 系统调用处理函数

// 系统调用实现函数
void syscall_write(const char *str);       // 系统调用：输出字符串
void syscall_exit(int status);             // 系统调用：退出进程并返回状态码
void syscall_read(char *buffer, size_t size); // 系统调用：读取数据到缓冲区
void* syscall_alloc();                     // 系统调用：分配内存
void syscall_free(void* ptr);              // 系统调用：释放内存
void syscall_sleep(int milliseconds);      // 系统调用：使进程休眠指定毫秒数

// 系统时钟相关函数
unsigned int get_system_ticks(void);       // 返回当前系统时钟计数（以tick为单位）

#endif // SYSCALL_H
