#include "video.h"
#include "string.h"
#include "utils.h"
#include "syscall.h"
#include "kernel.h"
#include "memory.h"

int clock_command = 0;

void kmain(void)
{
    clear_screen();
    log_message("Bootloader started kernel successfully.");
    memory_init();
    log_message("Memory system initialized.");
    init_thread_queue();
    log_message("Thread Queue initialized.");
    init_interrupts();
    log_message("Interrupt system initialized.");
    log_message("Welcome to YuMe OS.(Author: Xu Pan, Yu Liang and Yuhao Fang of HFUT)");
    while (1)
    {
        print_string("> ");

        char input[100];
        get_input(input, 100);
        if (strcmp(input, "hello") == 0)
        {
            log_message("Hello YuMe OS!");
            continue; 
        }
        else if (strcmp(input, "help") == 0)
        {
            log_message("Available commands: ");
            log_message("1. hello - Screen output test");
            log_message("2. echo - Say what you say");
            log_message("3. <expression> - Evaluate a mathematical expression");
            continue; 
        }
        else if (strcmp(input, "clear") == 0)
        {
            clear_screen();
            continue;
        }
        else if (strncmp(input, "count ", 6) == 0)
        {
            clock_command = atoi(input + 6);
            io_wait();
            if(clock_command == 0){
                continue;
            }
        }
        else if (strncmp(input, "echo ", 5) == 0)
        {
            print_string("\nYuMe: ");
            print_string(input + 5);
            print_string("\n");
            continue;
        }
        int result = eval_expression(input);

        // Print the result
        print_string("\nYuMe: ");
        print_int(result);
        print_string("\n");
    }
}
void get_input(char *buffer, int max_len)
{
    int i = 0;     // 当前输入字符的索引
    char c;        // 用于存储从键盘获取的字符

    // 循环读取字符，直到按下回车键或达到最大长度限制
    while (i < max_len - 1)
    {
        c = getchar(); // 从终端获取一个字符

        if (c == '\n')
        {          // 如果是换行符（回车键）
            break; // 结束输入
        }

        print_char(c); // 使用 `print_char` 显示输入的字符

        buffer[i++] = c; // 将字符存入缓冲区
    }

    buffer[i] = '\0'; // 在字符串末尾添加空字符作为终止符
}

void print_int(int num)
{
    char str[20];        // 临时字符串缓冲区，存储整数转换后的字符串
    itoa(num, str, 10);  // 将整数转换为字符串（十进制）
    print_string(str);   // 打印转换后的字符串
}

char getchar(void)
{
    static int shift_pressed = 0; // 记录Shift键是否被按下
    unsigned char scancode, ascii; // 扫描码和对应的ASCII码

    while (1)
    {
        // 等待键盘输出缓冲区有数据
        while ((inb(KEYBOARD_STATUS_PORT) & KEYBOARD_OUTPUT_FULL) == 0)
            ;
        scancode = inb(KEYBOARD_DATA_PORT); // 从键盘数据端口读取扫描码

        // 检查是否为Shift键按下或释放
        if (scancode == KEY_SHIFT)
        {
            shift_pressed = 1; // 标记Shift键被按下
            continue;
        }
        else if (scancode == KEY_SHIFT_RELEASE)
        {
            shift_pressed = 0; // 标记Shift键被释放
            continue;
        }

        // 处理普通键按下的情况（扫描码高位不为1）
        if (!(scancode & 0x80))
        {
            ascii = scancode_to_ascii(scancode, shift_pressed); // 转换扫描码为ASCII码
            if (ascii)
            {
                return ascii; // 返回有效的ASCII字符
            }
        }
    }
}

// 带Shift支持的扫描码到ASCII码的转换
char scancode_to_ascii(unsigned char scancode, int shift_pressed)
{
    // 普通（未按下Shift键）字符映射表
    static char ascii_table[] = {
        0, 0, '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '-', '=', 0,
        0, 'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', '[', ']', '\n',
        0, 'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ';', '\'', '`',
        0, '\\', 'z', 'x', 'c', 'v', 'b', 'n', 'm', ',', '.', '/', 0,
        '*', 0, ' '};

    // 按下Shift键时的字符映射表
    static char ascii_table_shift[] = {
        0, 0, '!', '@', '#', '$', '%', '^', '&', '*', '(', ')', '_', '+', 0,
        0, 'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P', '{', '}', '\n',
        0, 'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', ':', '"', '~',
        0, '|', 'Z', 'X', 'C', 'V', 'B', 'N', 'M', '<', '>', '?', 0,
        '*', 0, ' '};

    // 根据Shift键状态选择对应的映射表
    if (scancode < sizeof(ascii_table))
    {
        return shift_pressed ? ascii_table_shift[scancode] : ascii_table[scancode];
    }
    return 0; // 返回0表示无效字符
}
