#include "video.h"
#include "io.h"

// 屏幕尺寸和视频内存
#define SCREEN_WIDTH 80
#define SCREEN_HEIGHT 25
#define VIDEO_MEMORY (volatile unsigned char*) 0xB8000

// 光标位置
unsigned int cursor_x = 0;
unsigned int cursor_y = 0;

// 颜色
unsigned char current_color = 0x02;  // 黑绿

// 设置光标位置
void set_cursor_position(unsigned int x, unsigned int y) {
    unsigned short cursor_position = y * SCREEN_WIDTH + x;

    // Write the cursor position to the VGA registers
    outb(0x3D4, 0x0F); // Set the low byte of the cursor position
    outb(0x3D5, cursor_position & 0xFF);
    outb(0x3D4, 0x0E); // Set the high byte of the cursor position
    outb(0x3D5, (cursor_position >> 8) & 0xFF);
}

// 打印字符
void print_char(char c) {
    if (c == '\n') {
        cursor_x = 0;
        cursor_y++;
        if (cursor_y >= SCREEN_HEIGHT) {
            cursor_y = SCREEN_HEIGHT - 1;
        }
    } else {
        unsigned int index = (cursor_y * SCREEN_WIDTH + cursor_x) * 2;
        *(VIDEO_MEMORY + index) = c;
        *(VIDEO_MEMORY + index + 1) = current_color;
        cursor_x++;
        if (cursor_x >= SCREEN_WIDTH) {
            cursor_x = 0;
            cursor_y++;
            if (cursor_y >= SCREEN_HEIGHT) {
                cursor_y = SCREEN_HEIGHT - 1;
            }
        }
    }

    set_cursor_position(cursor_x, cursor_y);
}

// 打印字符串
void print_string(const char *str) {
    while (*str) {
        print_char(*str++);
    }
}

// 清屏
void clear_screen(void) {
    unsigned int i = 0;
    while (i < SCREEN_WIDTH * SCREEN_HEIGHT) {
        *(VIDEO_MEMORY + i * 2) = ' ';
        *(VIDEO_MEMORY + i * 2 + 1) = current_color;
        i++;
    }
    cursor_x = 0;
    cursor_y = 0;
    set_cursor_position(cursor_x, cursor_y);
}

void set_text_color(unsigned char color) {
    current_color = color;
}

void set_cyan_color(void) {
    set_text_color(0x03);  // Cyan
}

void set_red_color(void) {
    set_text_color(0x04);  // Red
}

void set_green_color(void) {
    set_text_color(0x02);  // Green
}
